6.120   os16: strtok(3)

NOME

    strtok - string token, ovvero estrazione di pezzi da una stringa

SINTASSI

    #include <string.h>
    char *strtok (char *restrict <string>, const char *restrict <delim>);

DESCRIZIONE

    La funzione <strtok()> serve a suddividere una stringa in unita`, definite
    token, specificando un elenco di caratteri da intendere come delimitatori,
    in una seconda stringa. La funzione va usata in fasi successive, fornendo
    solo inizialmente la stringa da suddividere che continua poi a essere
    utilizzata se al suo posto viene fornito il puntatore nullo. La funzione
    restituisce, di volta in volta, il puntatore alla sottostringa contenente
    l'unita` individuata, oppure il puntatore nullo, se non puo` trovarla.

    La funzione deve tenere memoria di un puntatore in modo persistente e deve
    isolare le unita` modificando la stringa originale, inserendo il carattere
    nullo di terminazione alla fine delle unita` individuate.

    Quando la funzione viene chiamata indicando al posto della stringa da
    scandire il puntatore nullo, l'insieme dei delimitatori puo` essere diverso
    da quello usato nelle fasi precedenti.

    Per comprendere lo scopo della funzione viene utilizzato lo stesso esempio
    che appare nel documento ISO/IEC 9899:TC2, al paragrafo 7.21.5.7, con
    qualche piccola modifica per poterlo rendere un programma autonomo:

    #include <stdio.h>
    #include <string.h>
    int
    main (void)
    {
        char str[] = "?a???b,,,#c";
        char *t;

        t = strtok (str, "?");              // t punta all'unita` "a"
        printf ("strtok: \"%s\"\n", t);
        t = strtok (NULL, ",");             // t punta all'unita` "??b"
        printf ("strtok: \"%s\"\n", t);
        t = strtok (NULL, "#,");            // t punta all'unita` "c"
        printf ("strtok: \"%s\"\n", t);
        t = strtok (NULL, "?");             // t e` un puntatore nullo
        printf ("strtok: \"%s\"\n", t);

        return 0;
    }

    Avviando il programma si ottiene quanto gia` descritto dai commenti inseriti
    nel codice:

    strtok: "a"
    strtok: "??b"
    strtok: "c"
    strtok: "(null)"

    Cio` che avviene nell'esempio puo` essere schematizzato come segue.
    Inizialmente la stringa str ha in memoria l'aspetto seguente:

      .--- str[]
      |
      v
    .---.---.---.---.---.---.---.---.---.---.---.---.
    | ? | a | ? | ? | ? | b | , | , | , | # | c |\0 |
    `---`---`---`---`---`---`---`---`---`---`---`---'

    Dopo la prima chiamata della funzione <strtok()> la stringa risulta
    alterata e il puntatore ottenuto raggiunge la lettera a:

          .--- t = strtok (str, "?");
          |
          v
    .---.---.---.---.---.---.---.---.---.---.---.---.
    | ? | a |\0 | ? | ? | b | , | , | , | # | c |\0 |
    `---`---`---`---`---`---`---`---`---`---`---`---'

    Dopo la seconda chiamata della funzione, in cui si usa il puntatore nullo
    per richiedere una scansione ulteriore della stringa originale, si ottiene
    un nuovo puntatore che, questa volta, inizia a partire dal quarto
    carattere, rispetto alla stringa originale, dal momento che il terzo e` gia`
    stato sovrascritto da un carattere nullo:

                  .--- t = strtok (NULL, ",");
                  |
                  v
    .---.---.---.---.---.---.---.---.---.---.---.---.
    | ? | a |\0 | ? | ? | b |\0 | , | , | # | c |\0 |
    `---`---`---`---`---`---`---`---`---`---`---`---'

    La penultima chiamata della funzione <strtok()> raggiunge la lettera c che
    e` anche alla fine della stringa originale:

          .--- t = strtok (NULL, "#,");
          |
          `-----------------------------------.
                                              |
                                              v
    .---.---.---.---.---.---.---.---.---.---.---.---.
    | ? | a |\0 | ? | ? | b |\0 | , | , | # | c |\0 |
    `---`---`---`---`---`---`---`---`---`---`---`---'

    L'ultimo tentativo di chiamata della funzione non puo` dare alcun esito,
    perche' la stringa originale si e` gia` conclusa.

VALORE RESTITUITO

    La funzione restituisce il puntatore al prossimo "pezzo", oppure NULL se
    non ce ne sono piu`.

FILE SORGENTI

    lib/string.h [13.11]

    lib/string/strtok.c [13.11.23]

VEDERE ANCHE

    memchr(3) [6.68], strchr(3) [6.105], strpbrk(3) [6.116], strspn(3) [6.118].

